%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    %Description
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This m-file replicates all the analysis published by KSS 
%forthcoming in ECTA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                     %External Packages                 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
path(path,'~/matlab_bgl/'); %note: matlab BGL obtained from http://www.mathworks.com/matlabcentral/fileexchange/10922
path(path,'codes'); %this contains the main LeaveOut Routines.
warning('off', 'all') 
cd codes;    
path(path,'CMG'); %Combinatorial multigrid obtained here: https://www.cs.cmu.edu/~jkoutis/cmg.html
MakeCMG;
cd ..
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                  %PARALLEL CODING ENVIRONMENT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
try
delete(gcp('nocreate'))
pool=parpool(32,'IdleTimeout', Inf)
%pool=parpool(str2num(getenv('SLURM_CPUS_PER_TASK')),'IdleTimeout', Inf);
%[a b]=system('sinfo -p low -o "%C"');
%cores=str2num([b(19) b(20)]);
%cores=min(cores,200)
%pool = parpool('dcs', cores);
%pool = parpool('dcs', 150);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                  		%TABLE 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%We start by reading the .csv file created in the build part and then
%find for each sample (all,young,old,1999-2001 as well as all Veneto 1996-2001)
%the largest connected set, the leave one out connected set, the leave two out
%connected set.

%The m-file `table_1' saves associated CSV files that will be read for the
%subsequent analysis.

%Warning: finding a leave two out connected set is a computationally challenging task,
%as described in the Computational Appendix.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 1 == 1  
  	TABELLA = zeros(15,4);
  		
    for pp=1:3 % pp>3 construct the samples used for table A1         
	
			if pp == 1
            namecity=['All_Veneto'];   
            end
            
            if pp == 2
            namecity=['young_All_Veneto'];   
            end 
            
            if pp == 3
            namecity=['old_All_Veneto'];   
            end
            
            if pp == 4
            namecity=['Appendix_All_Veneto'];   
            end
            
            if pp == 5
            namecity=['Appendix_young_All_Veneto'];   
            end 
            
            if pp == 6
            namecity=['Appendix_old_All_Veneto'];   
            end
            
            %Data, Log Calls
            if pp <= 3
            namesrc=['/scratch/public/leave_out/replication/' namecity '_1999and2001_T_equal_2.csv'];
            end
            
            if pp >3
            namesrc	=['/scratch/public/leave_out/replication/All_Veneto_1996and2001_T_greater_2.csv']; 
            end
            
            placelog='logs/';
            c= fix(clock);
            namelog=[namecity 'TABLE_1_' num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_' num2str(c(6))];
			logname=['logs/' namelog  '.log'];
			system(['rm ' logname])
			diary(logname)
			nameFile=namelog;
			placeFile='results/';
			filename=[placeFile nameFile];
			logname=['logs/' namelog  '.log'];
			system(['rm ' logname])
			diary(logname)
	
			%Import data
			data=importdata(namesrc);		
			id=data(:,1);			
			firmid=data(:,2);
			year=data(:,3);
			y=data(:,4);
			
			%Calculate birth year
			birth_YEAR = year-data(:,6);
			old		   = (birth_YEAR<=1964);
			clear data
			
			if pp == 5 %here I do the cut directly in matlab
			sel=old==0;
			id=id(sel);
			y=y(sel);
			firmid=firmid(sel);
			old=old(sel);
			birth_YEAR=birth_YEAR(sel);
			year=year(sel);
			end
			
			if pp == 6
			sel=old==1;
			id=id(sel);
			y=y(sel);
			firmid=firmid(sel);
			old=old(sel);
			birth_YEAR=birth_YEAR(sel);
			year=year(sel);
			end
			
			%Output table
            TABELLA(:,pp)=table_1(y,id,firmid,[old birth_YEAR year],namecity);   
            diary off
	end
	
	%Close
	disp('TABLE 1')	
	num2str(TABELLA(:,1:3))
	dlmwrite('tables/table1.csv', TABELLA, 'delimiter', '\t', 'precision', 16);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                  		%TABLE 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Here we compute the variance decomposition as shown in Table 2 of KSS.

%The code imports a given leave one out file saved by table_1 above and then performs
%the calculations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 1 == 1  
  	TABELLA = zeros(16,3);
  	index_table = 1;
  		
    for pp= 1:3
    
			if pp == 1
            namecity=['All_Veneto'];   
            end
            
            if pp == 2
            namecity=['young_All_Veneto'];   
            end 
            
            if pp == 3
            namecity=['old_All_Veneto'];   
            end
            
            for lll = 1:1 % lll=2 would show results for leave two out sample.
				
				%Data, Log Calls
				placelog='logs/';
				c= fix(clock);
				namelog=[namecity 'TABLE_2_Leave' num2str(lll) '_' num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_' num2str(c(6))];
				nameFile=namelog;
				placeFile='results/';
				filename=[placeFile nameFile];
				logname=['logs/' namelog  '.log'];
				system(['rm ' logname])
				diary(logname)

				%Import data
				namesrc=['/scratch/public/leave_out/replication/' namecity 'CS_Leave' num2str(lll) '.csv'];
				data=importdata(namesrc);
				id=data(:,4);
				firmid=data(:,5);
				y=data(:,3);
				year=data(:,end);
				old=data(:,end-2);
				birth = data(:,end-1);
				clear data

				%KSS Options
				leave_out_level='obs';
				andrews_estimates=1;
				eigen_diagno=0;
				subsample_llr_fit=0;
				restrict_movers=0;
				resid_controls=1;
				age = year-birth;
				controls=[year==2001];
				do_SE=0;
				type_of_algorithm='exact';
				epsilon=0.01;
	
				%Output table
				TABELLA(:,index_table)=table_2(y,id,firmid,leave_out_level,year,controls,resid_controls,andrews_estimates,eigen_diagno,subsample_llr_fit,restrict_movers,do_SE, type_of_algorithm,epsilon,filename);
				num2str(TABELLA)
				index_table = index_table +  1;
				diary off	
			
			end
						
	end
	
	%Close
	disp('TABLE 2')	
	num2str(TABELLA)
	dlmwrite('tables/table2.csv', TABELLA, 'delimiter', '\t', 'precision', 16);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                  		%TABLE 3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Here we read the All Veneto leave one out connected set, and then perform
%a post-estimation analysis where estimated firm effects are projected on 
%some observables characteristics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Table 3 - Panel A: 
if 1 == 1   	
			%File to work with:
			namecity=['All_Veneto'];
			
			%Leave One out or Leave 2 Out set:
			lll=1;
			
			%Log File
			placelog='logs/';
			c= fix(clock);
			namelog=[namecity 'Table_3_Panel_A' '_' num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_' num2str(c(6))];
			logname=[placelog namelog '.log'];
			system(['rm ' logname])
			diary(logname)

			%How to name the results
			nameFile=namelog;
			placeFile='results/';
			filename=[placeFile nameFile];

			%Import data
			namesrc=['/scratch/public/leave_out/replication/' namecity 'CS_Leave' num2str(lll) '.csv'];
			data=importdata(namesrc);
			id=data(:,1);
			firmid=data(:,2);
			y=data(:,3);
			year=data(:,end);
			old=data(:,end-2);
			clear data

			%KSS Options
			leave_out_level='obs';
			andrews_estimates=0;
			eigen_diagno=0;
			subsample_llr_fit=0;
			restrict_movers=0;
			resid_controls=1;
			controls=old;
			do_SE=0;
			type_of_algorithm='exact';
			epsilon=0.05;
	
			%Output Table
			TABELLA=table_3(y,id,firmid,leave_out_level,year,old,resid_controls,andrews_estimates,eigen_diagno,subsample_llr_fit,restrict_movers,do_SE, type_of_algorithm,epsilon,filename);
	
			%Close
			disp('TABLE 3')	
			num2str(TABELLA)
			dlmwrite('tables/table3.csv', TABELLA, 'delimiter', '\t', 'precision', 16); 
			diary off
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                  	%FIGURE 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Here we analyze whether firm effects of young workers
%are systematically different from the ones of old workers.

%A more detailed description of the steps taken to perform such analysis
%is provided in the code "kss_cck".

%The final graph shown in KSS is computed using STATA, see figure1.do
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 1 == 1    		
			namecity=['All_Veneto']; 
						
			%Log File
			placelog='logs/';
			c= fix(clock);
			namelog=[namecity 'Figure_1_' '_' num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_' num2str(c(6))];
			logname=[placelog namelog '.log'];
			system(['rm ' logname])
			diary(logname)
			nameFile=namelog;
			placeFile='results/';
			filename=[placeFile nameFile];
			
			%Call the file 
			lll=1;
			namesrc=['/scratch/public/leave_out/replication/' namecity 'CS_Leave' num2str(lll) '.csv'];
			data=importdata(namesrc);
			
			id=data(:,4);
			firmid=data(:,5);
			y=data(:,3);
			year=data(:,end);
			old=data(:,end-2);
			clear data
	
			%Run
			leave_out_level='obs';
			epsilon=0.01;
			type_of_algorithm='exact';
			[ttest,V, student]= kss_cck(old,y,id,firmid,year,[year==2001], leave_out_level, type_of_algorithm,epsilon,filename)
	
						
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                  	%TABLE 4
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Here we conduct inference on the variance of firm effects.

%Code is going to read Leave 1 out and Leave 2 out samples created
%initially by the table_1 routine.
 
%Our split-sample inference procedure described in the Computational
%Appendix is contained inside the function ``leave_out_FD".
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 1 == 1 
  	TABELLA = zeros(26,2,3);    
    
    for pp=1:3 
    
			if pp == 1
            namecity=['All_Veneto'];   
            end
            
            if pp == 2
            namecity=['young_All_Veneto'];   
            end 
            
            if pp == 3
            namecity=['old_All_Veneto'];   
            end
            
            for lll = 1:2
				%Data, Log Calls
				placelog='logs/';
				c= fix(clock);
				namelog=[namecity 'TABLE_4_Leave' num2str(lll) '_' num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_' num2str(c(6))];
				nameFile=namelog;
				placeFile='results/';
				filename=[placeFile nameFile];
				logname=['logs/' namelog  '.log'];
				system(['rm ' logname])
				diary(logname)

				%Import data
				namesrc=['/scratch/public/leave_out/replication/' namecity 'CS_Leave' num2str(lll) '.csv'];
				data=importdata(namesrc);
				id=data(:,1);
				firmid=data(:,2);
				y=data(:,3);
				year=data(:,end);
				age=year-data(:,end-1);
				controls = [year==2001];
				clear data

				%KSS Options
				leave_out_level='obs';
				andrews_estimates=0;
				eigen_diagno=1;
				eigen_fast=1;
				do_montecarlo=0;
				leave_2_out=0;
				restrict_movers=0;
				resid_controls=0;
				do_SE=0;
				type_of_algorithm='exact';
				epsilon=0.005;
				
				%Output table
				TABELLA(:,lll,pp) = table_4(y,id,firmid,leave_out_level,leave_2_out,controls,type_of_algorithm,eigen_diagno,eigen_fast,do_montecarlo,restrict_movers,epsilon,filename);
				diary off
			end	
	end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                  	%TABLE 5
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%MC tables. We warn the user that computation of Table 5 requires a lot of time.
%For this reason, by default, the replication package does not compute this table.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 0 == 1  
	
	%Start by creating the DGP by looking at the pooled sample, leave out out
	%Now run the full MC
	TABELLA = zeros(10,3);    
    
    for pp=1:3
    		conto=1;
    
			if pp == 3
            namecity=['old_All_Veneto'];   
            end
            
            if pp == 2
            namecity=['young_All_Veneto'];   
            end 
            
            if pp == 1
            namecity=['All_Veneto'];   
            end

            for lll = 1:2
				%Data, Log Calls
				placelog='logs/';
				c=fix(clock);
				namelog=[namecity 'TABLE_5_Leave' num2str(lll) '_' num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_' num2str(c(6))];
				nameFile=namelog;
				placeFile='results/';
				filename=[placeFile nameFile];
				logname=['logs/' namelog  '.log'];
				system(['rm ' logname])
				diary(logname)

				%Import data
				namesrc=['/scratch/public/leave_out/replication/' namecity 'CS_Leave' num2str(lll) '.csv'];
				data=importdata(namesrc);
				id=data(:,4);
				firmid=data(:,2);
				y=data(:,3);
				year=data(:,end);
				controls=year==2001;
				clear data

				%KSS Options
				leave_out_level='obs';
				andrews_estimates=0;
				eigen_diagno=1;
				eigen_fast=1;
				do_montecarlo=1;
				leave_2_out=0;
				restrict_movers=0;
				resid_controls=0;
				do_SE=0;
				
				if pp ==3
				type_of_algorithm='exact';
				end
				
				if pp < 3
				type_of_algorithm='JLL'; %we will run the JLA approximation for pooled and young sample to speed-up computation.
				end
				
				epsilon=0.005;

				%Output table
				TABELLA(:,conto) = table_5(y,id,firmid,leave_out_level,leave_2_out,controls,type_of_algorithm,eigen_diagno,eigen_fast,do_montecarlo,restrict_movers,epsilon,filename);
				conto=conto+1;
				diary off
			end	
	end
	
	%Close
	disp('TABLE 5')	
	num2str(TABELLA)
	dlmwrite('tables/table5_new_MC.csv', TABELLA, 'delimiter', '\t', 'precision', 16);

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                  	%TABLE A1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 1 == 1  
			
			for pp=1:6
			
  			if pp == 1
            namecity=['Appendix_All_VenetoCS_Leave1']; 
            end
  			
  			if pp == 2
            namecity=['Appendix_All_VenetoCS_Leave2'];  
            end           

			if pp == 3
            namecity=['Appendix_young_All_VenetoCS_Leave1'];  
            end
  			
  			if pp == 4;
            namecity=['Appendix_young_All_VenetoCS_Leave2'];  
            end           

			if pp == 5
            namecity=['Appendix_old_All_VenetoCS_Leave1'];  
            end
  			
  			if pp == 6
            namecity=['Appendix_old_All_VenetoCS_Leave2'];  
            end           

            
            %Data, Log Calls
            namesrc=['/scratch/public/leave_out/replication/' namecity '.csv']; %where original data is'        
            placelog='logs/';
            c= fix(clock);
            namelog=[namecity 'TABLE_A1' '_' num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_' num2str(c(6))];	
			nameFile=namelog;
			placeFile='results/';
			filename=[placeFile nameFile];
			logname=['logs/' namelog  '.log'];
			system(['rm ' logname])
			diary(logname)
	
			%Import data
			data=importdata(namesrc);
			id=data(:,4);
			firmid=data(:,2);
			y=data(:,3);
			year=data(:,end);
			age = year-data(:,end-1);
			clear data
			
			%KSS Options
			leave_out_level='obs';
			andrews_estimates=0;
			eigen_diagno=0;
			subsample_llr_fit=0;
			restrict_movers=1;
			resid_controls=1;
			do_SE=0;
			type_of_algorithm='JLL'; %note: our function that runs the JLA algorithm does not fix a random seed number.
			epsilon=0.001;
		
			
			%Controls
			[~,~,controls]  = unique(year);
			controls 	    = [sparse((1:size(y,1))',controls',1,size(y,1),max(controls))];
			controls(:,end) = [];
			controls		= [controls ((age-40)/40).^2 ((age-40)/40).^3];
				
			%Output Table
			TABELLA=table_app1(y,id,firmid,leave_out_level,year,controls,resid_controls,andrews_estimates,eigen_diagno,subsample_llr_fit,restrict_movers,do_SE, type_of_algorithm,epsilon,filename);

			%Close
			num2str(TABELLA)
			diary off
			end
						
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%					COMPUTATIONAL APPENDIX						 %

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Note: by default, the replication package does not compute the following
%additional results. Simply change line 589 to 1 == 1 if the user
%wants to replicate this part as well.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        			  %Figure 2.1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Here we work with a dataset that spans Veneto from 1990-2001. 

%We use different windows to compare the accuracy and speed of the JLA
%algorithm.

%In this appendix, we use the function leave_out_KSS. leave_out_KSS
%is an improved version of leave_out_COMPLETE that primarily substitutes
%the input "epsilon" with a scale parameter which directly controls the # of simulations in 
%the JLA algorithm. 

%The Github page dedicated KSS will contain new updates, improvements and extensions
%of the JLA algorithm so make sure to check https://github.com/rsaggio87/LeaveOutTwoWay for updates. 

%The final figure is then derived in Stata, using FigureB1.do.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 0 == 1 

			namesrc			=['/scratch/public/leave_out/All_Veneto_1990unbalanced2001_T_greater_2LMSW_test_data.csv']; %%this is src data which spans several years (1990-2001), At the end decided to focus on only 4 windows
			data_orig       = importdata(namesrc);							
			for mm = 1:2
					
					if mm == 1
					type_of_algorithm='JLL';
					end
				
					if mm == 2
					type_of_algorithm='exact';
					end
				
			
					for pp = 1:4
				
					if pp == 1
					time_use=[2000;2001]
					end
				
					if pp == 2
					time_use=[1999;2000;2001]
					end
				
					if pp == 3
					time_use=[1998;1999;2000;2001]
					end

					if pp == 4
					time_use=[1997;1998;1999;2000;2001]
					end
				
					sel				= ismember(data_orig(:,6),time_use);
					data			= data_orig(sel,:);
					sel				= data(:,2)==1; %%just keeping log daily wages observations in the stacked data that has wages,days and earnings.
					data			= data(sel,:);
					index_obs       = data(:,1);
					outcome_index   = data(:,2);
					id              = data(:,3);
					firmid          = data(:,4);
					firmid_year     = data(:,5);
					year            = data(:,6);
					y               = data(:,8);
					
					if pp >=2 %collapse down to match means.
					[~,~,id]		= unique(id);
					[~,~,firmid]	= unique(firmid);
					[~,~,match_id]  = unique([id firmid],'rows','stable');
					id				= accumarray(match_id,id,[],@(x)mean(x));
					firmid			= accumarray(match_id,firmid,[],@(x)mean(x));
					y				= accumarray(match_id,y,[],@(x)mean(x));
					year			= accumarray(match_id,year,[],@(x)min(x));	
					
					%need to resort
					[~,~,SS] 		= unique([id year],'rows','stable');
					[~,IX]			= sort(SS);
					y				= y(IX);
					id				= id(IX);
					firmid			= firmid(IX);
					year		 	= year(IX);		
					end
				
			
					%KSS Options
					leave_out_level='obs';
					andrews_estimates=0;
					eigen_diagno=0;
					subsample_llr_fit=0;
					restrict_movers=0;
					resid_controls=0;
					do_SE=0;
					controls = [];
					
					if mm == 1
						for ss = 1:2
					
							if ss == 1
							scale = 500;
							end
					
							if ss == 2
							scale = 2500;
							end
				
							%Log File
							placelog='logs/';
							c= fix(clock);
							namelog=['Table_B1_COLLAPSED_PP_' num2str(pp) '_' type_of_algorithm '_scale' num2str(scale) '_'  num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_' num2str(c(6))];
							logname=[placelog namelog '.log'];
							system(['rm ' logname])
							diary(logname)

							%How to name the results
							nameFile=namelog;
							placeFile='results/';
							filename=[placeFile nameFile];
				
							%Run
							[one] = leave_out_KSS(y,id,firmid,leave_out_level,year,controls,resid_controls,andrews_estimates,eigen_diagno,subsample_llr_fit,restrict_movers,do_SE, type_of_algorithm,scale,filename); 
							diary off
						end
					end
					
					if mm == 2
							%Log File
							placelog='logs/';
							c= fix(clock);
							namelog=['Figure_1_COMPUTATIONAL_APPENDIX_COLLAPSED_PP_' num2str(pp) '_' type_of_algorithm '_' num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_' num2str(c(6))];
							logname=[placelog namelog '.log'];
							system(['rm ' logname])
							diary(logname)

							%How to name the results
							nameFile=namelog;
							placeFile='results/';
							filename=[placeFile nameFile];
				
							%Run
							[one] = leave_out_KSS(y,id,firmid,leave_out_level,year,controls,resid_controls,andrews_estimates,eigen_diagno,subsample_llr_fit,restrict_movers,do_SE, type_of_algorithm,scale,filename);
							diary off
					
				 	end		
			end
	   end			
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        			  %Figure 2.2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Here we scale our JLA routine for a very large dataset of the size
%considered in CHK (2013). This is done by considering replicas of
%a given sample, as described in the Appendix.

%We use different windows to compare the accuracy and speed of the JLA
%algorithm.

%The figure is then derived in Stata, using FigureB2.do.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 	1 == 1 
				
			for rr = [1 5 10 15 20]
			
			%Specify number of replicas and input data.
			total_n_of_replicas=rr;
            namecity = ['All_Veneto'];   
          	lll 	 = 1;
            
			%Import data
			namesrc		= ['/scratch/public/leave_out/replication/' namecity 'CS_Leave' num2str(lll) '.csv'];
			data			= importdata(namesrc);
			id_VERI     	= data(:,4);
			firmid_VERI 	= data(:,5);
			[~,~,id]    	= unique(data(:,1));
			[~,~,firmid]    = unique(data(:,2));
			y				= data(:,3);
			year			= data(:,end);
			
			%Relabel
			id_orig			= id;
			id_VERI_series  = id_VERI;
			firmid_orig		= firmid;
			year_orig		= year;
			y_orig			= y;
			NT				= size(data,1);
			cluster_firm 	= 1.*ones(size(data,1),1);
			cluster_worker	= 1.*ones(size(data,1),1);
			
			%give me the movers
			gcs = [NaN; id(1:end-1)];
			gcs = id~=gcs;
			lagfirmid=[NaN; firmid(1:end-1)];
			lagfirmid(gcs==1)=NaN; %%first obs for each worker
			stayer=(firmid==lagfirmid);
			stayer(gcs==1)=1;
			stayer=accumarray(id,stayer);
			T=accumarray(id,1);
			stayer=T==stayer;
			movers=stayer~=1;
			movers=movers(id);
			id_movers		= id(movers);
			[~,~,id_movers_norm]=unique(id_movers);
			
			disp('Total # of workers starting')
			max(id)
			disp('Total # of firms starting')
			max(firmid)
			disp('Total # of movers starting')
			max(id_movers_norm)
			
			
			%parametric bootstrap to obtain "true" worker,firm effects
			NT				= size(y,1);
			J				= max(firmid);
			N				= max(id);
			D				= sparse(1:NT,id',1);
			F				= sparse(1:NT,firmid',1);
			S				= speye(J-1);
   			S				= [S;sparse(-zeros(1,J-1))];  %N+JxN+J-1 restriction matrix 
			X				= [D F*S year==2001];
			xx				= X'*X;
  			Lchol			= ichol(xx,struct('type','ict','droptol',1e-2,'diagcomp',.1));
   			xy				= X'*y;
   			b				= pcg(xx,xy,1e-10,1000,Lchol,Lchol');
   			y				= y-X(:,N+J:end)*b(N+J:end);
   			X				= [D F*S];
   			xx				= X'*X;
  			Lchol			= ichol(xx,struct('type','ict','droptol',1e-2,'diagcomp',.1));
   			xy				= X'*y;
   			b				= pcg(xx,xy,1e-10,1000,Lchol,Lchol');
   			psi				= b(N+1:end);
   			alpha			= b(1:N);
   			disp('PI variance of firm effects')
   			sigma_2_psi_AKM =var(F*S*psi)
   			sigma_2_psi	    = 0.0239535626707183;
			psi				= psi*sqrt(sigma_2_psi/sigma_2_psi_AKM);
			disp('KSS variance of firm effects')
   			sigma_2_psi 	= var(F*S*psi)
			
			%Create the replicas
			for cc = 2:total_n_of_replicas
			cluster_firm 	= [cluster_firm;   cc.*ones(size(data,1),1)];
			cluster_worker 	= [cluster_worker; cc.*ones(size(data,1),1)];
			id				= [id; id_orig];
			firmid			= [firmid;firmid_orig];
			y				= [y;y_orig];
			year			= [year;year_orig];
			id_VERI			= [id_VERI; id_VERI_series];
			end
			
			%Create the circles
			if total_n_of_replicas > 1
			s				= randi(total_n_of_replicas-1,max(id),1);
			D				= sparse(1:size(y,1)',id',1,size(y,1),max(id));
			s				= D*s;
			cluster_new		= (1+mod(cluster_firm,s)).*(cluster_firm+s>total_n_of_replicas)+(cluster_firm+s).*(cluster_firm+s<=total_n_of_replicas);
			
			%Select the workers that will form the bridges
			bridges			= randi(max(id_movers_norm),round(0.10*max(id_movers_norm)),1);
			bridges			= id_movers(bridges);
			sel				= ismember(id,bridges);
			sel 			= (sel > 0 & year == min(year));
			aiuto			= [id year firmid cluster_firm cluster_new s];
			aiuto(id==bridges(1),:)
			cluster_firm(sel)= cluster_new(sel);
			aiuto			= [id year firmid cluster_firm cluster_new s];
			aiuto(id==bridges(1),:)
			clear aiuto
			end
			
			%New ids
			id_old			= id;
			firmid_old		= firmid;
			[~,~,id] 		= unique([cluster_worker id],'rows','stable');
			[~,~,firmid] 	= unique([cluster_firm   firmid],'rows','stable');
			
			disp('Total # of workers after replicas')
			max(id)
			disp('Total # of firms after replicas')
			max(firmid)
			
			%Need to resort data
			[~,~,SS] 		= unique([id year cluster_worker],'rows','stable');
			[~,IX]			= sort(SS);
			y				= y(IX);
			id				= id(IX);
			firmid			= firmid(IX);
			year		 	= year(IX);
			cluster_firm    = cluster_firm(IX);
			cluster_worker  = cluster_worker(IX);
			id_old			= id_old(IX);
			firmid_old		= firmid_old(IX);
			id_VERI			= id_VERI(IX);
			
			clear id_orig firmid_orig  y_orig year_orig cluster_firm cluster_worker data bridges
			
			%Span the firm and worker effects
			NT				= size(y,1);
			D				= sparse(1:NT,id_old',1);
			F				= sparse(1:NT,firmid_old',1);
			psi				= [psi;0];
			fe				= F*psi;
			pe				= D*alpha;
			disp('---VERIFY:True variance of firm effects')
			var(fe)
			
			%Now get the sigma_i from original DGP created for MC
			namesrc			= ['results/All_VenetoDGP_CREATION1_2019_7_10_7_35_40DGP_values.csv']; %% the user needs to replace this with the file created by leave_out_FD when computing Table 5, for the Pooled Leave 1 out sample. This file contains ``the true" sigma_i to be read by the parametric bootstrap routine outlined below.
			data			= importdata(namesrc);
			sigma_i			= data(:,3);
			id_POOL			= data(:,1);
			sel				= ismember(id_VERI,id_POOL);
			id_movers		= id_old(sel);
			id_movers     	= unique(id_movers);
			sigma_i			= sparse(id_movers,1,sigma_i/2,max(id_old),1);
			sigma_i			= D*sigma_i;
			num2str([(1:100)' sigma_i(1:100) firmid_old(1:100) id_old(1:100) id(1:100)]) %to make sure that the data has the right structure.
			clear data id_POOL sel id_movers	
			
			%Simulate the y
			y 				= pe + fe + sqrt(sigma_i).*randn(NT,1);
			
			%KSS Options
			leave_out_level='obs';
			andrews_estimates=0;
			eigen_diagno=0;
			subsample_llr_fit=0;
			restrict_movers=0;
			resid_controls=0;
			type_of_algorithm='JLL';
			controls = [];
			do_SE=0;
			
			%Log File
			for ss = 1:2
			if ss == 2 
			scale = 500;
			end
			
			if ss == 1
			scale = 250;
			end
			
			placelog='logs/';
			c= fix(clock);
			namelog=['Figure_2_Computational_Appednix_REPLICAS_' num2str(total_n_of_replicas) '_SCALE' num2str(scale) '_' type_of_algorithm '_' num2str(c(1)) '_' num2str(c(2)) '_' num2str(c(3)) '_' num2str(c(4)) '_' num2str(c(5)) '_' num2str(c(6))];
			logname=[placelog namelog '.log'];
			system(['rm ' logname])
			diary(logname)

			%How to name the results
			nameFile=namelog;
			placeFile='/scratch/public/leave_out/';
			filename=[placeFile nameFile];
			
			[one] = leave_out_KSS(y,id,firmid,leave_out_level,year,controls,resid_controls,andrews_estimates,eigen_diagno,subsample_llr_fit,restrict_movers,do_SE, type_of_algorithm,scale,filename);
			end
			end

end
delete(pool)