********************************************************************************
			
								*NOTES*
			
********************************************************************************
*This do-file takes the raw-data from the VHW files and constructs a person-year
*panel.

*Build follows closely CHK (2013). Specifically, we select job (and firm) that paid the most 
*a worker in a given year by combining the three sources of data contained in the VWH
*files. The output of this code is going to be a person-year file of all workers employed 
*in the Veneto region from 1985-2001. 

*Note: the VHW description file online mentions that the data should pertain only to the
*provinces of Vicenza and Treviso. As far as we can tell, this is not true for the data
*that was submitted to us which pretty clearly seems to encompass the entire Veneto region.
*This is shown in the initial tabulation of Section 5 of this do-file.

local dir_tmp_aux "$dir_tmp"
********************************************************************************
			
						*SECTION 1: BUILD PY PANEL *
			
********************************************************************************
if 1 == 1 { 
*LOAD
	use "src/contr",replace

*SMALL CROSSWALK
	rename matr_az firmid
	rename cod_pgr id
	rename anno year

*Let's just keep data from 1985
	keep if year>=1985
	
*Kick out the duplicated rows
   duplicates drop id year firmid gior retrib03, force          
	
*SELECT JOB THAT PAID THE MOST IN A GIVEN YEAR
	gsort id year -retrib03
	by id year: gen tmp=_n
	bys id year: gen total_jobs_in_year=_N
	tab total_jobs_in_year
	preserve
	collapse (mean) total_jobs_in_year, by(id year)
	tabstat total_jobs_in_year, by(year)
	restore
	keep if tmp==1
	
	
*GENERATE (GROSS) LOG DAILY WAGES
	sum gior_r,d //this would be full of zeros if i were to use data-pre 1985
	sum sett_u,d
	sum sett_r,d //more than 50 percent just report 52...  
	gen daily_wage=retrib03/gior_r
	gen log_dailywages=log(retrib03/gior_r)
	gen weekly_wage=retrib03/sett_r
	gen log_weekly_wage=log(weekly_wage)
	
*Summing up (people at fdbr thinks that wages are already in real terms...)
	tabstat log_dailywages log_weekly_wage retrib03, by(year)
 	
*SAVE AND MOVE ON
	keep id firmid year gior_r daily_wage log_dailywages log_weekly_wage total_jobs_in_year qualif contrat tipo_rap livello
	save "`dir_tmp_aux'/py_start",replace
}	
********************************************************************************
			
					*SECTION 2: CLEAN WORKERS' FILE *
			
********************************************************************************
if 1 == 1 { 
*LOAD
	use "src/anagr",replace
*CROSSWALK
	rename cod_pgr id
	rename anno_n birth
	rename sesso sex
	rename naz foreign 
	rename prov_n prov_birth
	rename prov_r prov_res
	tab prov_birth
	tab prov_res
	
*SAVE AND MOVE ON
	keep id birth sex foreign prov_birth prov_res 	
	save "`dir_tmp_aux'/p_start",replace	
}	
********************************************************************************
			
					*SECTION 3: CLEAN FIRMS' FILE *
			
********************************************************************************
if 1 == 1 { 
*LOAD
	use "src/azien",replace
	
*SMALL CROSSWALK
	rename matr_az firmid
	rename comune comune_azienda
	tab prov,sort
	
*SAVE AND MOVE ON
	keep firmid comune_azienda csc prov
	save "`dir_tmp_aux'/f_start",replace
} 	
********************************************************************************
			
							*SECTION 4: MERGE*
			
********************************************************************************
if 1 == 1 { 
*LOAD
	use "`dir_tmp_aux'/py_start",replace

*FIRM FILE
	merge m:1 firmid using "`dir_tmp_aux'/f_start", gen(match_f)
	keep if match_f==3
	
*WORKER FILE
	merge m:1 id using "`dir_tmp_aux'/p_start", gen(match_p)
	keep if match_p==3
	drop match_f match_p
*SAVE AND MOVE ON
	save "`dir_tmp_aux'/py_start2",replace
}	
********************************************************************************
			
				*SECTION 5: PREPARE FILE FOR ESTIMATION*
			
********************************************************************************
*LOAD
	use "`dir_tmp_aux'/py_start2",replace
	
*WRITE	
	capture file close myfile	
	file open myfile using "logs/Observations_Cleaning.txt", text write replace
	
*verify the sampling frame, how many person year observations fall in the two cities? to me this looks like the entire region of Veneto. 
	tab prov,sort
	
*is there a worker that was never employed (top-job) neither in Treviso and Vicenza?
	gen in_file=0
	replace in_file=1 if prov=="VI" | prov=="TV"
	bys id: egen in_file_ind=max(in_file)
	drop in_file
	gen in_file=0
	replace in_file=1 if in_file_ind==0
	sum in_file,d
	drop in_file in_file_ind

*is there a worker that was never employed (top-job) in Veneto?
	gen in_file=0
	replace in_file=1 if prov=="BL" | prov=="PD" | prov=="RO" | prov=="TV" | prov=="VE" | prov=="VR" | prov=="VI"
	bys id: egen in_file_ind=max(in_file)
	drop in_file
	gen in_file=0
	replace in_file=1 if in_file_ind==0
	sum in_file,d // approx 11%, this could be due to py collapse. Gonna drop these dudes.	
	
*just person year obs in Veneto
	keep if prov=="BL" | prov=="PD" | prov=="RO" | prov=="TV" | prov=="VE" | prov=="VR" | prov=="VI"
	tab prov, sort
	
*just py obs from 18-64
	gen age=year-birth
	drop if age>=64
	drop if age<=18
	
distinct id
local ndist = r(ndistinct)
local nn    = r(N)
file write myfile %9s "After dropping by age: number of individuals: " %7.0f (`ndist') %9s ", number of observations: " %7.0f (`nn') _n	
	
*small daily wages
	gen very_small_wage=0
	replace very_small_wage=1 if daily_wage<=5
	bys id: egen very_small=mean(very_small_wage)
	drop if very_small>0
	drop very_small_wage very_small
	
*0 days worked
	gen zero_days=0 
	replace zero_days=1 if daily_wage==.
	bys id: egen zero_days_aux=mean(zero_days)
	drop if zero_days_aux>0
	drop zero_days_aux zero_days
	
distinct id
local ndist = r(ndistinct)
local nn    = r(N)
file write myfile %9s "After dropping by small wages and 0 days worked: number of individuals: " %7.0f (`ndist') %9s ", number of observations: " %7.0f (`nn') _n		
	

	
*too big wage changes
	xtset id year
	gen change_wage=log_dailywages-L1.log_dailywages
	sum change_wage,d
	gen change_wage_big=0
	replace change_wage_big=1 if change_wage>=1 & change_wage!=. | change_wage<=-1 
	bys id: egen change_wage_big_id=mean(change_wage_big)
	drop if change_wage_big_id>0
	drop change_wage_big_id  change_wage_big change_wage
	
distinct id
local ndist = r(ndistinct)
local nn    = r(N)
file write myfile %9s "After dropping too big wage changes: number of individuals: " %7.0f (`ndist') %9s ", number of observations: " %7.0f (`nn') _n		
		
*no public sector	
	gen ind_codes_short=floor(csc/10000)
	tab ind_codes_short
	gen ever_public=0
	replace ever_public=1 if ind_codes_short==2 | ind_codes_short==3
	bys id: egen ever_public_id=mean(ever_public)
	drop if ever_public_id>0
	drop ever_public_id ever_public
	
distinct id
local ndist = r(ndistinct)
local nn    = r(N)
file write myfile %9s "After dropping no public: number of individuals: " %7.0f (`ndist') %9s ", number of observations: " %7.0f (`nn') _n			
	
*too many jobs in a year?	
	tab total_jobs_in_year 
	gen too_many_jobs=0
	replace too_many_jobs=1 if total_jobs_in_year>=10
	bys id: egen too_many=mean(too_many_jobs)
	drop if too_many>0
	drop too_many_jobs too_many	
	
	
distinct id
local ndist = r(ndistinct)
local nn    = r(N)
file write myfile %9s "After dropping too many jobs: number of individuals: " %7.0f (`ndist') %9s ", number of observations: " %7.0f (`nn') _n		
	
	
*no gender?
	gen no_gender=0
	replace no_gender=1 if sex=="X"
	bys id: egen no_gender_aux=mean(no_gender)
	drop if no_gender_aux>0
	drop no_gender_aux no_gender
	
distinct id
local ndist = r(ndistinct)
local nn    = r(N)
file write myfile %9s "After dropping no gender: number of individuals: " %7.0f (`ndist') %9s ", number of observations: " %7.0f (`nn') _n		
			
	
*Last clean: Gender, Start Age, foreign, 
	bys id: egen start_year=min(year)
	bys id: egen real_age_at_entry=min(age)
	gen female=0
	replace female=1 if sex=="F"
	rename foreign foreign_s
	gen foreign=0
	replace foreign=1 if foreign_s!="" | foreign_s!="I"
	
	gen log_days=log(gior_r)
	gen log_earnings=log(daily_wage*gior_r)
	
*SAVE
	describe
	distinct id
	distinct firmid
	save "`dir_tmp_aux'/py_final_1985_2001_veneto_only",replace	
	
