********************************************************************************

							*DESCRIPTION*

********************************************************************************
*Here we take the baseline file created by our build routine, we impose some restrictions
*(years, age, etc) to then create a separate .csv file that is going to used for the 
*analysis (carried out in Matlab).

*The global $provGLOBAl tells the code whether to keep the entire region of the Veneto (provGLOBAl==1)
*or to focus just on the young workers (provGLOBAl==2) or the old workers (provGLOBAl==3).

*To construct Table A1 of KSS we used a file that comprises all 
*observations in Veneto from 1996-2001. To construct this baseline dataset, the user should
*set provGLOBAl==4.

********************************************************************************
local randomness = $provGLOBAl
local oldWorker=40
local Min_year=1999
local Max_year=2001
local T_2_rest=1
local balanc_restr=0
local other_notes=""

if `T_2_rest'== 1 {
local type_of_panel="T_equal_2"
}

if `T_2_rest'== 0 {
local type_of_panel="T_greater_2"
}

if `balanc_restr'== 1 {
local avverbio="balanced"
}

if `balanc_restr'== 0 {
local avverbio="and"
}
********************************************************************************

							*CALL INPUTS*

********************************************************************************
local dir_tmp_aux "$dir_tmp"
local place "$cityName"
local place_log "$cityName_log"
local spec_name="VENETO_PUBLIC_DATA"
local city_time="`place_log'_`Min_year'`avverbio'`Max_year'_`type_of_panel'`other_notes'"
local filename "`spec_name'_`city_time'_`other_notes'"
local path_tmp "`dir_tmp_aux'/"
********************************************************************************

					*SECTION 1: CALL ORIGINAL FILE*

********************************************************************************
if 1 == 1 {
	use "`dir_tmp_aux'/py_final_1985_2001_veneto_only",replace	
********************************************************************************

					*SECTION 2: SETTING THE DATA*

********************************************************************************
*baseline characteristcs
	egen idnew=group(id)
	egen firmidnew=group(firmid)
	sum year
	sum idnew
	sum firmidnew
	bys firmid year: gen firmsize=_N
	replace firmsize=log(firmsize)
	bys firmid: egen min_year=min(year)
	gen firm_age=year-min_year
	egen type_worker=group(year real_age_at_entry female)
	sum age if year == 1999,d
	sum female if year==1999 & age>=40
	sum female if year==1999 & age<40
	
	
*timeout for controls
	drop if female==.
	drop if age==.
	drop if year==.
	gen age_norm=(age-40)/40
	gen age_norm2=age_norm^2
	gen age_norm3=age_norm^3

*young workers only?
	if $provGLOBAl == 2 {
	gen birth_YEAR = year-age
	keep if birth_YEAR>=1965 & birth_YEAR<=1983
	sum age if year == 1999
	drop birth_YEAR
	}

*old workers only?	
	if $provGLOBAl == 3 {
	gen birth_YEAR = year-age
	keep if birth_YEAR<=1964
	sum age if year == 1999
	drop birth_YEAR
	}
	
	if $provGLOBAl == 4 {
	local Min_year=1996
	local Max_year=2001
	local T_2_rest=0
	local type_of_panel="T_greater_2"	
	local city_time="`place_log'_`Min_year'`avverbio'`Max_year'_`type_of_panel'`other_notes'"
	local filename "`spec_name'_`city_time'_`other_notes'"	
	}
	
*do we want T=2?
	if `T_2_rest' == 1 {
	keep if year==`Min_year' | year==`Max_year'
	bys id: gen TTT=_N
	}
	
*length of the panel?
	if `T_2_rest' == 0 {
	keep if year>=`Min_year' & year<=`Max_year'
	bys id: gen TTT=_N
	}	
	
*do we want a balanced panel?
if `balanc_restr' == 1{ 
	sum TT,d
	keep if TTT==r(max)
	}	

*auxiliary to run the code	
	xtset id year
	bys id: gen lagfirmid=firmidnew[_n-1]
	egen provN=group(prov)
	tabulate provN prov
********************************************************************************

				 *SECTION 3: SUBMIT TO MATLAB*

********************************************************************************
*save the associated src file
	local save_filename "`dir_tmp_aux'/`city_time'"
	xtset id year
	replace lagfirmid=-9 if lagfirmid==.
	sum $controls_to_use
	keep id firmidnew year provN log_dailywages $controls_to_use 
	export delimited id firmidnew year log_dailywages provN $controls_to_use using `save_filename', replace novarnames nolabel
	save `save_filename',replace


